/*
** 2000-05-29
**
** The author disclaims copyright to this source code.  In place of
** a legal notice, here is a blessing:
**
**    May you do good and not evil.
**    May you find forgiveness for yourself and forgive others.
**    May you share freely, never taking more than you give.
**
*************************************************************************
** Driver template for the LEMON parser generator.
**
** The "lemon" program processes an LALR(1) input grammar file, then uses
** this template to construct a parser.  The "lemon" program inserts text
** at each "%%" line.  Also, any "P-a-r-s-e" identifer prefix (without the
** interstitial "-" characters) contained in this template is changed into
** the value of the %name directive from the grammar.  Otherwise, the content
** of this template is copied straight through into the generate parser
** source file.
**
** The following is the concatenation of all %include directives from the
** input grammar file:
*/
#include <stdio.h>
/************ Begin %include sections from the grammar ************************/
#line 2 "./grammar.lemon"

#include "config.h"

#include <assert.h>

#include "dfilter-int.h"
#include "syntax-tree.h"
#include "sttype-range.h"
#include "sttype-test.h"
#include "sttype-function.h"
#include "sttype-set.h"
#include "drange.h"

#include "grammar.h"

#ifdef _WIN32
#pragma warning(disable:4671)
#endif

/* End of C code */
#line 49 "grammar.c"
/**************** End of %include directives **********************************/
/* These constants specify the various numeric values for terminal symbols
** in a format understandable to "makeheaders".  This section is blank unless
** "lemon" is run with the "-m" command-line option.
***************** Begin makeheaders token definitions *************************/
/**************** End makeheaders token definitions ***************************/

/* The next sections is a series of control #defines.
** various aspects of the generated parser.
**    YYCODETYPE         is the data type used to store the integer codes
**                       that represent terminal and non-terminal symbols.
**                       "unsigned char" is used if there are fewer than
**                       256 symbols.  Larger types otherwise.
**    YYNOCODE           is a number of type YYCODETYPE that is not used for
**                       any terminal or nonterminal symbol.
**    YYFALLBACK         If defined, this indicates that one or more tokens
**                       (also known as: "terminal symbols") have fall-back
**                       values which should be used if the original symbol
**                       would not parse.  This permits keywords to sometimes
**                       be used as identifiers, for example.
**    YYACTIONTYPE       is the data type used for "action codes" - numbers
**                       that indicate what to do in response to the next
**                       token.
**    DfilterTOKENTYPE     is the data type used for minor type for terminal
**                       symbols.  Background: A "minor type" is a semantic
**                       value associated with a terminal or non-terminal
**                       symbols.  For example, for an "ID" terminal symbol,
**                       the minor type might be the name of the identifier.
**                       Each non-terminal can have a different minor type.
**                       Terminal symbols all have the same minor type, though.
**                       This macros defines the minor type for terminal
**                       symbols.
**    YYMINORTYPE        is the data type used for all minor types.
**                       This is typically a union of many types, one of
**                       which is DfilterTOKENTYPE.  The entry in the union
**                       for terminal symbols is called "yy0".
**    YYSTACKDEPTH       is the maximum depth of the parser's stack.  If
**                       zero the stack is dynamically sized using realloc()
**    DfilterARG_SDECL     A static variable declaration for the %extra_argument
**    DfilterARG_PDECL     A parameter declaration for the %extra_argument
**    DfilterARG_STORE     Code to store %extra_argument into yypParser
**    DfilterARG_FETCH     Code to extract %extra_argument from yypParser
**    YYERRORSYMBOL      is the code number of the error symbol.  If not
**                       defined, then do no error processing.
**    YYNSTATE           the combined number of states.
**    YYNRULE            the number of rules in the grammar
**    YY_MAX_SHIFT       Maximum value for shift actions
**    YY_MIN_SHIFTREDUCE Minimum value for shift-reduce actions
**    YY_MAX_SHIFTREDUCE Maximum value for shift-reduce actions
**    YY_MIN_REDUCE      Maximum value for reduce actions
**    YY_ERROR_ACTION    The yy_action[] code for syntax error
**    YY_ACCEPT_ACTION   The yy_action[] code for accept
**    YY_NO_ACTION       The yy_action[] code for no-op
*/
#ifndef INTERFACE
# define INTERFACE 1
#endif
/************* Begin control #defines *****************************************/
#define YYCODETYPE unsigned char
#define YYNOCODE 43
#define YYACTIONTYPE unsigned char
#define DfilterTOKENTYPE stnode_t*
typedef union {
  int yyinit;
  DfilterTOKENTYPE yy0;
  test_op_t yy2;
  drange_node* yy29;
  GSList* yy81;
} YYMINORTYPE;
#ifndef YYSTACKDEPTH
#define YYSTACKDEPTH 100
#endif
#define DfilterARG_SDECL dfwork_t *dfw;
#define DfilterARG_PDECL ,dfwork_t *dfw
#define DfilterARG_FETCH dfwork_t *dfw = yypParser->dfw
#define DfilterARG_STORE yypParser->dfw = dfw
#define YYNSTATE             29
#define YYNRULE              43
#define YY_MAX_SHIFT         28
#define YY_MIN_SHIFTREDUCE   59
#define YY_MAX_SHIFTREDUCE   101
#define YY_MIN_REDUCE        102
#define YY_MAX_REDUCE        144
#define YY_ERROR_ACTION      145
#define YY_ACCEPT_ACTION     146
#define YY_NO_ACTION         147
/************* End control #defines *******************************************/

/* Define the yytestcase() macro to be a no-op if is not already defined
** otherwise.
**
** Applications can choose to define yytestcase() in the %include section
** to a macro that can assist in verifying code coverage.  For production
** code the yytestcase() macro should be turned off.  But it is useful
** for testing.
*/
#ifndef yytestcase
# define yytestcase(X)
#endif


/* Next are the tables used to determine what action to take based on the
** current state and lookahead token.  These tables are used to implement
** functions that take a state number and lookahead value and return an
** action integer.
**
** Suppose the action integer is N.  Then the action is determined as
** follows
**
**   0 <= N <= YY_MAX_SHIFT             Shift N.  That is, push the lookahead
**                                      token onto the stack and goto state N.
**
**   N between YY_MIN_SHIFTREDUCE       Shift to an arbitrary state then
**     and YY_MAX_SHIFTREDUCE           reduce by rule N-YY_MIN_SHIFTREDUCE.
**
**   N between YY_MIN_REDUCE            Reduce by rule N-YY_MIN_REDUCE
**     and YY_MAX_REDUCE
**
**   N == YY_ERROR_ACTION               A syntax error has occurred.
**
**   N == YY_ACCEPT_ACTION              The parser accepts its input.
**
**   N == YY_NO_ACTION                  No such action.  Denotes unused
**                                      slots in the yy_action[] table.
**
** The action table is constructed as a single large table named yy_action[].
** Given state S and lookahead X, the action is computed as either:
**
**    (A)   N = yy_action[ yy_shift_ofst[S] + X ]
**    (B)   N = yy_default[S]
**
** The (A) formula is preferred.  The B formula is used instead if:
**    (1)  The yy_shift_ofst[S]+X value is out of range, or
**    (2)  yy_lookahead[yy_shift_ofst[S]+X] is not equal to X, or
**    (3)  yy_shift_ofst[S] equal YY_SHIFT_USE_DFLT.
** (Implementation note: YY_SHIFT_USE_DFLT is chosen so that
** YY_SHIFT_USE_DFLT+X will be out of range for all possible lookaheads X.
** Hence only tests (1) and (2) need to be evaluated.)
**
** The formulas above are for computing the action when the lookahead is
** a terminal symbol.  If the lookahead is a non-terminal (as occurs after
** a reduce action) then the yy_reduce_ofst[] array is used in place of
** the yy_shift_ofst[] array and YY_REDUCE_USE_DFLT is used in place of
** YY_SHIFT_USE_DFLT.
**
** The following are the tables generated in this section:
**
**  yy_action[]        A single table containing all actions.
**  yy_lookahead[]     A table containing the lookahead for each entry in
**                     yy_action.  Used to detect hash collisions.
**  yy_shift_ofst[]    For each state, the offset into yy_action for
**                     shifting terminals.
**  yy_reduce_ofst[]   For each state, the offset into yy_action for
**                     shifting non-terminals after a reduce.
**  yy_default[]       Default action for each state.
**
*********** Begin parsing tables **********************************************/
#define YY_ACTTAB_COUNT (129)
static const YYACTIONTYPE yy_action[] = {
 /*     0 */    85,   86,   89,   90,   87,   88,   92,   93,   91,  103,
 /*    10 */    99,  102,    4,    3,   25,   17,   21,   16,    7,   24,
 /*    20 */    20,    2,   27,   26,   69,   70,  146,   15,   12,   61,
 /*    30 */    62,   77,   25,   23,   22,   19,    1,   24,    2,   27,
 /*    40 */    26,   69,   70,   76,   18,   75,   13,    5,    6,    9,
 /*    50 */    10,   80,   19,    1,   14,   12,   61,   62,   97,   25,
 /*    60 */    65,   12,   61,   62,   24,   25,    4,    3,   79,   78,
 /*    70 */    24,   64,   12,   61,   62,  117,   25,   27,   26,   69,
 /*    80 */    70,   24,    3,  116,   28,   12,   61,   62,  115,   25,
 /*    90 */    19,  104,   98,  101,   24,   27,   26,   69,   70,   27,
 /*   100 */    26,   69,   70,  104,  104,   95,   96,   94,   19,   25,
 /*   110 */    25,  104,   19,    8,   24,   24,  104,   11,   84,  100,
 /*   120 */   104,   25,  104,   25,  104,  104,   24,  104,   24,
};
static const YYCODETYPE yy_lookahead[] = {
 /*     0 */     3,    4,    5,    6,    7,    8,    9,   10,   11,    0,
 /*    10 */    32,    0,    1,    2,   36,   20,   21,   39,   35,   41,
 /*    20 */    23,   12,   13,   14,   15,   16,   30,   31,   32,   33,
 /*    30 */    34,   37,   36,   21,   22,   26,   27,   41,   12,   13,
 /*    40 */    14,   15,   16,   37,   38,   18,   19,   27,   24,   19,
 /*    50 */    17,   20,   26,   27,   31,   32,   33,   34,   28,   36,
 /*    60 */    31,   32,   33,   34,   41,   36,    1,    2,   20,   20,
 /*    70 */    41,   31,   32,   33,   34,   17,   36,   13,   14,   15,
 /*    80 */    16,   41,    2,   17,   31,   32,   33,   34,   17,   36,
 /*    90 */    26,   42,   28,   28,   41,   13,   14,   15,   16,   13,
 /*   100 */    14,   15,   16,   42,   42,   32,   32,   25,   26,   36,
 /*   110 */    36,   42,   26,   40,   41,   41,   42,   32,   33,   32,
 /*   120 */    42,   36,   42,   36,   42,   42,   41,   42,   41,
};
#define YY_SHIFT_USE_DFLT (129)
#define YY_SHIFT_COUNT    (28)
#define YY_SHIFT_MIN      (-5)
#define YY_SHIFT_MAX      (86)
static const short yy_shift_ofst[] = {
 /*     0 */     9,   26,   26,   26,   26,   64,   86,   86,   82,   86,
 /*    10 */    -5,   -3,   -3,   -5,   65,   11,   30,   12,   27,   20,
 /*    20 */    24,   31,   48,   49,   33,   58,   66,   71,   80,
};
#define YY_REDUCE_USE_DFLT (-23)
#define YY_REDUCE_COUNT (13)
#define YY_REDUCE_MIN   (-22)
#define YY_REDUCE_MAX   (87)
static const signed char yy_reduce_ofst[] = {
 /*     0 */    -4,   23,   29,   40,   53,  -22,   73,   85,   74,   87,
 /*    10 */     6,  -17,  -17,   -6,
};
static const YYACTIONTYPE yy_default[] = {
 /*     0 */   145,  145,  145,  145,  145,  145,  145,  145,  145,  145,
 /*    10 */   145,  126,  109,  145,  145,  145,  145,  125,  145,  145,
 /*    20 */   145,  145,  145,  124,  145,  114,  111,  110,  106,
};
/********** End of lemon-generated parsing tables *****************************/

/* The next table maps tokens (terminal symbols) into fallback tokens.
** If a construct like the following:
**
**      %fallback ID X Y Z.
**
** appears in the grammar, then ID becomes a fallback token for X, Y,
** and Z.  Whenever one of the tokens X, Y, or Z is input to the parser
** but it does not parse, the type of the token is changed to ID and
** the parse is retried before an error is thrown.
**
** This feature can be used, for example, to cause some keywords in a language
** to revert to identifiers if they keyword does not apply in the context where
** it appears.
*/
#ifdef YYFALLBACK
static const YYCODETYPE yyFallback[] = {
};
#endif /* YYFALLBACK */

/* The following structure represents a single element of the
** parser's stack.  Information stored includes:
**
**   +  The state number for the parser at this level of the stack.
**
**   +  The value of the token stored at this level of the stack.
**      (In other words, the "major" token.)
**
**   +  The semantic value stored at this level of the stack.  This is
**      the information used by the action routines in the grammar.
**      It is sometimes called the "minor" token.
**
** After the "shift" half of a SHIFTREDUCE action, the stateno field
** actually contains the reduce action for the second half of the
** SHIFTREDUCE.
*/
struct yyStackEntry {
  YYACTIONTYPE stateno;  /* The state-number, or reduce action in SHIFTREDUCE */
  YYCODETYPE major;      /* The major token value.  This is the code
                         ** number for the token at this stack level */
  YYMINORTYPE minor;     /* The user-supplied minor token value.  This
                         ** is the value of the token  */
};
typedef struct yyStackEntry yyStackEntry;

/* The state of the parser is completely contained in an instance of
** the following structure */
struct yyParser {
  yyStackEntry *yytos;          /* Pointer to top element of the stack */
#ifdef YYTRACKMAXSTACKDEPTH
  int yyhwm;                    /* High-water mark of the stack */
#endif
#ifndef YYNOERRORRECOVERY
  int yyerrcnt;                 /* Shifts left before out of the error */
#endif
  DfilterARG_SDECL                /* A place to hold %extra_argument */
#if YYSTACKDEPTH<=0
  int yystksz;                  /* Current side of the stack */
  yyStackEntry *yystack;        /* The parser's stack */
  yyStackEntry yystk0;          /* First stack entry */
#else
  yyStackEntry yystack[YYSTACKDEPTH];  /* The parser's stack */
#endif
};
typedef struct yyParser yyParser;

#ifndef NDEBUG
#include <stdio.h>
static FILE *yyTraceFILE = 0;
static char *yyTracePrompt = 0;
#endif /* NDEBUG */

#ifndef NDEBUG
/*
** Turn parser tracing on by giving a stream to which to write the trace
** and a prompt to preface each trace message.  Tracing is turned off
** by making either argument NULL
**
** Inputs:
** <ul>
** <li> A FILE* to which trace output should be written.
**      If NULL, then tracing is turned off.
** <li> A prefix string written at the beginning of every
**      line of trace output.  If NULL, then tracing is
**      turned off.
** </ul>
**
** Outputs:
** None.
*/
void DfilterTrace(FILE *TraceFILE, char *zTracePrompt){
  yyTraceFILE = TraceFILE;
  yyTracePrompt = zTracePrompt;
  if( yyTraceFILE==0 ) yyTracePrompt = 0;
  else if( yyTracePrompt==0 ) yyTraceFILE = 0;
}
#endif /* NDEBUG */

#ifndef NDEBUG
/* For tracing shifts, the names of all terminals and nonterminals
** are required.  The following table supplies these names */
static const char *const yyTokenName[] = {
  "$",             "TEST_AND",      "TEST_OR",       "TEST_EQ",     
  "TEST_NE",       "TEST_LT",       "TEST_LE",       "TEST_GT",     
  "TEST_GE",       "TEST_CONTAINS",  "TEST_MATCHES",  "TEST_BITWISE_AND",
  "TEST_NOT",      "FIELD",         "STRING",        "CHARCONST",   
  "UNPARSED",      "LBRACKET",      "RBRACKET",      "COMMA",       
  "INTEGER",       "COLON",         "HYPHEN",        "TEST_IN",     
  "LBRACE",        "RBRACE",        "FUNCTION",      "LPAREN",      
  "RPAREN",        "error",         "sentence",      "expr",        
  "entity",        "relation_test",  "logical_test",  "rel_op2",     
  "range",         "drnode",        "drnode_list",   "funcparams",  
  "setnode_list",  "range_body",  
};
#endif /* NDEBUG */

#ifndef NDEBUG
/* For tracing reduce actions, the names of all rules are required.
*/
static const char *const yyRuleName[] = {
 /*   0 */ "sentence ::= expr",
 /*   1 */ "sentence ::=",
 /*   2 */ "expr ::= relation_test",
 /*   3 */ "expr ::= logical_test",
 /*   4 */ "logical_test ::= expr TEST_AND expr",
 /*   5 */ "logical_test ::= expr TEST_OR expr",
 /*   6 */ "logical_test ::= TEST_NOT expr",
 /*   7 */ "logical_test ::= entity",
 /*   8 */ "entity ::= FIELD",
 /*   9 */ "entity ::= STRING",
 /*  10 */ "entity ::= CHARCONST",
 /*  11 */ "entity ::= UNPARSED",
 /*  12 */ "entity ::= range",
 /*  13 */ "range_body ::= FIELD",
 /*  14 */ "range_body ::= STRING",
 /*  15 */ "range_body ::= range",
 /*  16 */ "range ::= range_body LBRACKET drnode_list RBRACKET",
 /*  17 */ "drnode_list ::= drnode",
 /*  18 */ "drnode_list ::= drnode_list COMMA drnode",
 /*  19 */ "drnode ::= INTEGER COLON INTEGER",
 /*  20 */ "drnode ::= INTEGER HYPHEN INTEGER",
 /*  21 */ "drnode ::= COLON INTEGER",
 /*  22 */ "drnode ::= INTEGER COLON",
 /*  23 */ "drnode ::= INTEGER",
 /*  24 */ "relation_test ::= entity rel_op2 entity",
 /*  25 */ "relation_test ::= entity rel_op2 relation_test",
 /*  26 */ "rel_op2 ::= TEST_EQ",
 /*  27 */ "rel_op2 ::= TEST_NE",
 /*  28 */ "rel_op2 ::= TEST_GT",
 /*  29 */ "rel_op2 ::= TEST_GE",
 /*  30 */ "rel_op2 ::= TEST_LT",
 /*  31 */ "rel_op2 ::= TEST_LE",
 /*  32 */ "rel_op2 ::= TEST_BITWISE_AND",
 /*  33 */ "rel_op2 ::= TEST_CONTAINS",
 /*  34 */ "rel_op2 ::= TEST_MATCHES",
 /*  35 */ "relation_test ::= entity TEST_IN LBRACE setnode_list RBRACE",
 /*  36 */ "setnode_list ::= entity",
 /*  37 */ "setnode_list ::= setnode_list entity",
 /*  38 */ "entity ::= FUNCTION LPAREN funcparams RPAREN",
 /*  39 */ "entity ::= FUNCTION LPAREN RPAREN",
 /*  40 */ "funcparams ::= entity",
 /*  41 */ "funcparams ::= funcparams COMMA entity",
 /*  42 */ "expr ::= LPAREN expr RPAREN",
};
#endif /* NDEBUG */


#if YYSTACKDEPTH<=0
/*
** Try to increase the size of the parser stack.  Return the number
** of errors.  Return 0 on success.
*/
static int yyGrowStack(yyParser *p){
  int newSize;
  int idx;
  yyStackEntry *pNew;

  newSize = p->yystksz*2 + 100;
  idx = p->yytos ? (int)(p->yytos - p->yystack) : 0;
  if( p->yystack==&p->yystk0 ){
    pNew = malloc(newSize*sizeof(pNew[0]));
    if( pNew ) pNew[0] = p->yystk0;
  }else{
    pNew = realloc(p->yystack, newSize*sizeof(pNew[0]));
  }
  if( pNew ){
    p->yystack = pNew;
    p->yytos = &p->yystack[idx];
#ifndef NDEBUG
    if( yyTraceFILE ){
      fprintf(yyTraceFILE,"%sStack grows from %d to %d entries.\n",
              yyTracePrompt, p->yystksz, newSize);
    }
#endif
    p->yystksz = newSize;
  }
  return pNew==0;
}
#endif

/* Datatype of the argument to the memory allocated passed as the
** second argument to DfilterAlloc() below.  This can be changed by
** putting an appropriate #define in the %include section of the input
** grammar.
*/
#ifndef YYMALLOCARGTYPE
# define YYMALLOCARGTYPE size_t
#endif

/* Initialize a new parser that has already been allocated.
*/
static void DfilterInit(void *yypParser){
  yyParser *pParser = (yyParser*)yypParser;
#ifdef YYTRACKMAXSTACKDEPTH
  pParser->yyhwm = 0;
#endif
#if YYSTACKDEPTH<=0
  pParser->yytos = NULL;
  pParser->yystack = NULL;
  pParser->yystksz = 0;
  if( yyGrowStack(pParser) ){
    pParser->yystack = &pParser->yystk0;
    pParser->yystksz = 1;
  }
#endif
#ifndef YYNOERRORRECOVERY
  pParser->yyerrcnt = -1;
#endif
  pParser->yytos = pParser->yystack;
  pParser->yystack[0].stateno = 0;
  pParser->yystack[0].major = 0;
}

#ifndef Dfilter_ENGINEALWAYSONSTACK
/*
** This function allocates a new parser.
** The only argument is a pointer to a function which works like
** malloc.
**
** Inputs:
** A pointer to the function used to allocate memory.
**
** Outputs:
** A pointer to a parser.  This pointer is used in subsequent calls
** to Dfilter and DfilterFree.
*/
void *DfilterAlloc(void *(*mallocProc)(YYMALLOCARGTYPE)){
  yyParser *pParser;
  pParser = (yyParser*)(*mallocProc)( (YYMALLOCARGTYPE)sizeof(yyParser) );
  if( pParser ) DfilterInit(pParser);
  return pParser;
}
#endif /* Dfilter_ENGINEALWAYSONSTACK */


/* The following function deletes the "minor type" or semantic value
** associated with a symbol.  The symbol can be either a terminal
** or nonterminal. "yymajor" is the symbol code, and "yypminor" is
** a pointer to the value to be deleted.  The code used to do the
** deletions is derived from the %destructor and/or %token_destructor
** directives of the input grammar.
*/
static void yy_destructor(
  yyParser *yypParser,    /* The parser */
  YYCODETYPE yymajor,     /* Type code for object to destroy */
  YYMINORTYPE *yypminor   /* The object to be destroyed */
){
  DfilterARG_FETCH;
  switch( yymajor ){
    /* Here is inserted the actions which take place when a
    ** terminal or non-terminal is destroyed.  This can happen
    ** when the symbol is popped from the stack during a
    ** reduce or during error processing or when a parser is
    ** being destroyed before it is finished parsing.
    **
    ** Note: during a reduce, the only symbols destroyed are those
    ** which appear on the RHS of the rule, but which are *not* used
    ** inside the C code.
    */
/********* Begin destructor definitions ***************************************/
      /* TERMINAL Destructor */
    case 1: /* TEST_AND */
    case 2: /* TEST_OR */
    case 3: /* TEST_EQ */
    case 4: /* TEST_NE */
    case 5: /* TEST_LT */
    case 6: /* TEST_LE */
    case 7: /* TEST_GT */
    case 8: /* TEST_GE */
    case 9: /* TEST_CONTAINS */
    case 10: /* TEST_MATCHES */
    case 11: /* TEST_BITWISE_AND */
    case 12: /* TEST_NOT */
    case 13: /* FIELD */
    case 14: /* STRING */
    case 15: /* CHARCONST */
    case 16: /* UNPARSED */
    case 17: /* LBRACKET */
    case 18: /* RBRACKET */
    case 19: /* COMMA */
    case 20: /* INTEGER */
    case 21: /* COLON */
    case 22: /* HYPHEN */
    case 23: /* TEST_IN */
    case 24: /* LBRACE */
    case 25: /* RBRACE */
    case 26: /* FUNCTION */
    case 27: /* LPAREN */
    case 28: /* RPAREN */
{
#line 31 "./grammar.lemon"

	(void) dfw; /* Mark unused, similar to Q_UNUSED */
	stnode_free((yypminor->yy0));

#line 576 "grammar.c"
}
      break;
    case 31: /* expr */
    case 32: /* entity */
    case 33: /* relation_test */
    case 34: /* logical_test */
    case 36: /* range */
{
#line 40 "grammar.lemon"
stnode_free((yypminor->yy0));
#line 587 "grammar.c"
}
      break;
    case 37: /* drnode */
{
#line 57 "grammar.lemon"
drange_node_free((yypminor->yy29));
#line 594 "grammar.c"
}
      break;
    case 38: /* drnode_list */
{
#line 60 "grammar.lemon"
drange_node_free_list((yypminor->yy81));
#line 601 "grammar.c"
}
      break;
    case 39: /* funcparams */
{
#line 63 "grammar.lemon"
st_funcparams_free((yypminor->yy81));
#line 608 "grammar.c"
}
      break;
    case 40: /* setnode_list */
{
#line 66 "grammar.lemon"
set_nodelist_free((yypminor->yy81));
#line 615 "grammar.c"
}
      break;
/********* End destructor definitions *****************************************/
    default:  break;   /* If no destructor action specified: do nothing */
  }
}

/*
** Pop the parser's stack once.
**
** If there is a destructor routine associated with the token which
** is popped from the stack, then call it.
*/
static void yy_pop_parser_stack(yyParser *pParser){
  yyStackEntry *yytos;
  assert( pParser->yytos!=0 );
  assert( pParser->yytos > pParser->yystack );
  yytos = pParser->yytos--;
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sPopping %s\n",
      yyTracePrompt,
      yyTokenName[yytos->major]);
  }
#endif
  yy_destructor(pParser, yytos->major, &yytos->minor);
}

/*
** Clear all secondary memory allocations from the parser
*/
static void DfilterFinalize(void *p){
  yyParser *pParser = (yyParser*)p;
  while( pParser->yytos>pParser->yystack ) yy_pop_parser_stack(pParser);
#if YYSTACKDEPTH<=0
  if( pParser->yystack!=&pParser->yystk0 ) free(pParser->yystack);
#endif
}

#ifndef Dfilter_ENGINEALWAYSONSTACK
/*
** Deallocate and destroy a parser.  Destructors are called for
** all stack elements before shutting the parser down.
**
** If the YYPARSEFREENEVERNULL macro exists (for example because it
** is defined in a %include section of the input grammar) then it is
** assumed that the input pointer is never NULL.
*/
void DfilterFree(
  void *p,                    /* The parser to be deleted */
  void (*freeProc)(void*)     /* Function used to reclaim memory */
){
#ifndef YYPARSEFREENEVERNULL
  if( p==0 ) return;
#endif
  DfilterFinalize(p);
  (*freeProc)(p);
}
#endif /* Dfilter_ENGINEALWAYSONSTACK */

/*
** Return the peak depth of the stack for a parser.
*/
#ifdef YYTRACKMAXSTACKDEPTH
int DfilterStackPeak(void *p){
  yyParser *pParser = (yyParser*)p;
  return pParser->yyhwm;
}
#endif

/*
** Find the appropriate action for a parser given the terminal
** look-ahead token iLookAhead.
*/
static unsigned int yy_find_shift_action(
  yyParser *pParser,        /* The parser */
  YYCODETYPE iLookAhead     /* The look-ahead token */
){
  int i;
  int stateno = pParser->yytos->stateno;

  if( stateno>=YY_MIN_REDUCE ) return stateno;
  assert( stateno <= YY_SHIFT_COUNT );
  do{
    i = yy_shift_ofst[stateno];
    assert( iLookAhead!=YYNOCODE );
    i += iLookAhead;
    if( i<0 || i>=YY_ACTTAB_COUNT || yy_lookahead[i]!=iLookAhead ){
#ifdef YYFALLBACK
      YYCODETYPE iFallback;            /* Fallback token */
      if( iLookAhead<sizeof(yyFallback)/sizeof(yyFallback[0])
             && (iFallback = yyFallback[iLookAhead])!=0 ){
#ifndef NDEBUG
        if( yyTraceFILE ){
          fprintf(yyTraceFILE, "%sFALLBACK %s => %s\n",
             yyTracePrompt, yyTokenName[iLookAhead], yyTokenName[iFallback]);
        }
#endif
        assert( yyFallback[iFallback]==0 ); /* Fallback loop must terminate */
        iLookAhead = iFallback;
        continue;
      }
#endif
#ifdef YYWILDCARD
      {
        int j = i - iLookAhead + YYWILDCARD;
        if(
#if YY_SHIFT_MIN+YYWILDCARD<0
          j>=0 &&
#endif
#if YY_SHIFT_MAX+YYWILDCARD>=YY_ACTTAB_COUNT
          j<YY_ACTTAB_COUNT &&
#endif
          yy_lookahead[j]==YYWILDCARD && iLookAhead>0
        ){
#ifndef NDEBUG
          if( yyTraceFILE ){
            fprintf(yyTraceFILE, "%sWILDCARD %s => %s\n",
               yyTracePrompt, yyTokenName[iLookAhead],
               yyTokenName[YYWILDCARD]);
          }
#endif /* NDEBUG */
          return yy_action[j];
        }
      }
#endif /* YYWILDCARD */
      return yy_default[stateno];
    }else{
      return yy_action[i];
    }
  }while(1);
}

/*
** Find the appropriate action for a parser given the non-terminal
** look-ahead token iLookAhead.
*/
static int yy_find_reduce_action(
  int stateno,              /* Current state number */
  YYCODETYPE iLookAhead     /* The look-ahead token */
){
  int i;
#ifdef YYERRORSYMBOL
  if( stateno>YY_REDUCE_COUNT ){
    return yy_default[stateno];
  }
#else
  assert( stateno<=YY_REDUCE_COUNT );
#endif
  i = yy_reduce_ofst[stateno];
  assert( i!=YY_REDUCE_USE_DFLT );
  assert( iLookAhead!=YYNOCODE );
  i += iLookAhead;
#ifdef YYERRORSYMBOL
  if( i<0 || i>=YY_ACTTAB_COUNT || yy_lookahead[i]!=iLookAhead ){
    return yy_default[stateno];
  }
#else
  assert( i>=0 && i<YY_ACTTAB_COUNT );
  assert( yy_lookahead[i]==iLookAhead );
#endif
  return yy_action[i];
}

/*
** The following routine is called if the stack overflows.
*/
static void yyStackOverflow(yyParser *yypParser){
   DfilterARG_FETCH;
#ifndef NDEBUG
   if( yyTraceFILE ){
     fprintf(yyTraceFILE,"%sStack Overflow!\n",yyTracePrompt);
   }
#endif
   while( yypParser->yytos>yypParser->yystack ) yy_pop_parser_stack(yypParser);
   /* Here code is inserted which will execute if the parser
   ** stack every overflows */
/******** Begin %stack_overflow code ******************************************/
/******** End %stack_overflow code ********************************************/
   DfilterARG_STORE; /* Suppress warning about unused %extra_argument var */
}

/*
** Print tracing information for a SHIFT action
*/
#ifndef NDEBUG
static void yyTraceShift(yyParser *yypParser, int yyNewState){
  if( yyTraceFILE ){
    if( yyNewState<YYNSTATE ){
      fprintf(yyTraceFILE,"%sShift '%s', go to state %d\n",
         yyTracePrompt,yyTokenName[yypParser->yytos->major],
         yyNewState);
    }else{
      fprintf(yyTraceFILE,"%sShift '%s'\n",
         yyTracePrompt,yyTokenName[yypParser->yytos->major]);
    }
  }
}
#else
# define yyTraceShift(X,Y)
#endif

/*
** Perform a shift action.
*/
static void yy_shift(
  yyParser *yypParser,          /* The parser to be shifted */
  int yyNewState,               /* The new state to shift in */
  int yyMajor,                  /* The major token to shift in */
  DfilterTOKENTYPE yyMinor        /* The minor token to shift in */
){
  yyStackEntry *yytos;
  yypParser->yytos++;
#ifdef YYTRACKMAXSTACKDEPTH
  if( (int)(yypParser->yytos - yypParser->yystack)>yypParser->yyhwm ){
    yypParser->yyhwm++;
    assert( yypParser->yyhwm == (int)(yypParser->yytos - yypParser->yystack) );
  }
#endif
#if YYSTACKDEPTH>0
  if( yypParser->yytos>=&yypParser->yystack[YYSTACKDEPTH] ){
    yypParser->yytos--;
    yyStackOverflow(yypParser);
    return;
  }
#else
  if( yypParser->yytos>=&yypParser->yystack[yypParser->yystksz] ){
    if( yyGrowStack(yypParser) ){
      yypParser->yytos--;
      yyStackOverflow(yypParser);
      return;
    }
  }
#endif
  if( yyNewState > YY_MAX_SHIFT ){
    yyNewState += YY_MIN_REDUCE - YY_MIN_SHIFTREDUCE;
  }
  yytos = yypParser->yytos;
  yytos->stateno = (YYACTIONTYPE)yyNewState;
  yytos->major = (YYCODETYPE)yyMajor;
  yytos->minor.yy0 = yyMinor;
  yyTraceShift(yypParser, yyNewState);
}

/* The following table contains information about every rule that
** is used during the reduce.
*/
static const struct {
  YYCODETYPE lhs;         /* Symbol on the left-hand side of the rule */
  unsigned char nrhs;     /* Number of right-hand side symbols in the rule */
} yyRuleInfo[] = {
  { 30, 1 },
  { 30, 0 },
  { 31, 1 },
  { 31, 1 },
  { 34, 3 },
  { 34, 3 },
  { 34, 2 },
  { 34, 1 },
  { 32, 1 },
  { 32, 1 },
  { 32, 1 },
  { 32, 1 },
  { 32, 1 },
  { 41, 1 },
  { 41, 1 },
  { 41, 1 },
  { 36, 4 },
  { 38, 1 },
  { 38, 3 },
  { 37, 3 },
  { 37, 3 },
  { 37, 2 },
  { 37, 2 },
  { 37, 1 },
  { 33, 3 },
  { 33, 3 },
  { 35, 1 },
  { 35, 1 },
  { 35, 1 },
  { 35, 1 },
  { 35, 1 },
  { 35, 1 },
  { 35, 1 },
  { 35, 1 },
  { 35, 1 },
  { 33, 5 },
  { 40, 1 },
  { 40, 2 },
  { 32, 4 },
  { 32, 3 },
  { 39, 1 },
  { 39, 3 },
  { 31, 3 },
};

static void yy_accept(yyParser*);  /* Forward Declaration */

/*
** Perform a reduce action and the shift that must immediately
** follow the reduce.
*/
static void yy_reduce(
  yyParser *yypParser,         /* The parser */
  unsigned int yyruleno        /* Number of the rule by which to reduce */
){
  int yygoto;                     /* The next state */
  int yyact;                      /* The next action */
  yyStackEntry *yymsp;            /* The top of the parser's stack */
  int yysize;                     /* Amount to pop the stack */
  DfilterARG_FETCH;
  yymsp = yypParser->yytos;
#ifndef NDEBUG
  if( yyTraceFILE && yyruleno<(int)(sizeof(yyRuleName)/sizeof(yyRuleName[0])) ){
    yysize = yyRuleInfo[yyruleno].nrhs;
    fprintf(yyTraceFILE, "%sReduce [%s], go to state %d.\n", yyTracePrompt,
      yyRuleName[yyruleno], yymsp[-yysize].stateno);
  }
#endif /* NDEBUG */

  /* Check that the stack is large enough to grow by a single entry
  ** if the RHS of the rule is empty.  This ensures that there is room
  ** enough on the stack to push the LHS value */
  if( yyRuleInfo[yyruleno].nrhs==0 ){
#ifdef YYTRACKMAXSTACKDEPTH
    if( (int)(yypParser->yytos - yypParser->yystack)>yypParser->yyhwm ){
      yypParser->yyhwm++;
      assert( yypParser->yyhwm == (int)(yypParser->yytos - yypParser->yystack));
    }
#endif
#if YYSTACKDEPTH>0
    if( yypParser->yytos>=&yypParser->yystack[YYSTACKDEPTH-1] ){
      yyStackOverflow(yypParser);
      return;
    }
#else
    if( yypParser->yytos>=&yypParser->yystack[yypParser->yystksz-1] ){
      if( yyGrowStack(yypParser) ){
        yyStackOverflow(yypParser);
        return;
      }
      yymsp = yypParser->yytos;
    }
#endif
  }

  switch( yyruleno ){
  /* Beginning here are the reduction cases.  A typical example
  ** follows:
  **   case 0:
  **  #line <lineno> <grammarfile>
  **     { ... }           // User supplied code
  **  #line <lineno> <thisfile>
  **     break;
  */
/********** Begin reduce actions **********************************************/
        YYMINORTYPE yylhsminor;
      case 0: /* sentence ::= expr */
#line 143 "grammar.lemon"
{ dfw->st_root = yymsp[0].minor.yy0; }
#line 976 "grammar.c"
        break;
      case 1: /* sentence ::= */
#line 144 "grammar.lemon"
{ dfw->st_root = NULL; }
#line 981 "grammar.c"
        break;
      case 2: /* expr ::= relation_test */
      case 3: /* expr ::= logical_test */ yytestcase(yyruleno==3);
      case 8: /* entity ::= FIELD */ yytestcase(yyruleno==8);
      case 9: /* entity ::= STRING */ yytestcase(yyruleno==9);
      case 10: /* entity ::= CHARCONST */ yytestcase(yyruleno==10);
      case 11: /* entity ::= UNPARSED */ yytestcase(yyruleno==11);
      case 12: /* entity ::= range */ yytestcase(yyruleno==12);
      case 13: /* range_body ::= FIELD */ yytestcase(yyruleno==13);
      case 14: /* range_body ::= STRING */ yytestcase(yyruleno==14);
      case 15: /* range_body ::= range */ yytestcase(yyruleno==15);
#line 146 "grammar.lemon"
{ yylhsminor.yy0 = yymsp[0].minor.yy0; }
#line 995 "grammar.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 4: /* logical_test ::= expr TEST_AND expr */
#line 152 "grammar.lemon"
{
	yylhsminor.yy0 = stnode_new(STTYPE_TEST, NULL);
	sttype_test_set2(yylhsminor.yy0, TEST_OP_AND, yymsp[-2].minor.yy0, yymsp[0].minor.yy0);
}
#line 1004 "grammar.c"
  yy_destructor(yypParser,1,&yymsp[-1].minor);
  yymsp[-2].minor.yy0 = yylhsminor.yy0;
        break;
      case 5: /* logical_test ::= expr TEST_OR expr */
#line 158 "grammar.lemon"
{
	yylhsminor.yy0 = stnode_new(STTYPE_TEST, NULL);
	sttype_test_set2(yylhsminor.yy0, TEST_OP_OR, yymsp[-2].minor.yy0, yymsp[0].minor.yy0);
}
#line 1014 "grammar.c"
  yy_destructor(yypParser,2,&yymsp[-1].minor);
  yymsp[-2].minor.yy0 = yylhsminor.yy0;
        break;
      case 6: /* logical_test ::= TEST_NOT expr */
{  yy_destructor(yypParser,12,&yymsp[-1].minor);
#line 164 "grammar.lemon"
{
	yymsp[-1].minor.yy0 = stnode_new(STTYPE_TEST, NULL);
	sttype_test_set1(yymsp[-1].minor.yy0, TEST_OP_NOT, yymsp[0].minor.yy0);
}
#line 1025 "grammar.c"
}
        break;
      case 7: /* logical_test ::= entity */
#line 170 "grammar.lemon"
{
	yylhsminor.yy0 = stnode_new(STTYPE_TEST, NULL);
	sttype_test_set1(yylhsminor.yy0, TEST_OP_EXISTS, yymsp[0].minor.yy0);
}
#line 1034 "grammar.c"
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 16: /* range ::= range_body LBRACKET drnode_list RBRACKET */
#line 190 "grammar.lemon"
{
	yylhsminor.yy0 = stnode_new(STTYPE_RANGE, NULL);
	sttype_range_set(yylhsminor.yy0, yymsp[-3].minor.yy0, yymsp[-1].minor.yy81);

	/* Delete the list, but not the drange_nodes that
	 * the list contains. */
	g_slist_free(yymsp[-1].minor.yy81);
}
#line 1047 "grammar.c"
  yy_destructor(yypParser,17,&yymsp[-2].minor);
  yy_destructor(yypParser,18,&yymsp[0].minor);
  yymsp[-3].minor.yy0 = yylhsminor.yy0;
        break;
      case 17: /* drnode_list ::= drnode */
#line 200 "grammar.lemon"
{
	yylhsminor.yy81 = g_slist_append(NULL, yymsp[0].minor.yy29);
}
#line 1057 "grammar.c"
  yymsp[0].minor.yy81 = yylhsminor.yy81;
        break;
      case 18: /* drnode_list ::= drnode_list COMMA drnode */
#line 205 "grammar.lemon"
{
	yylhsminor.yy81 = g_slist_append(yymsp[-2].minor.yy81, yymsp[0].minor.yy29);
}
#line 1065 "grammar.c"
  yy_destructor(yypParser,19,&yymsp[-1].minor);
  yymsp[-2].minor.yy81 = yylhsminor.yy81;
        break;
      case 19: /* drnode ::= INTEGER COLON INTEGER */
#line 211 "grammar.lemon"
{
	yylhsminor.yy29 = drange_node_new();
	drange_node_set_start_offset(yylhsminor.yy29, stnode_value(yymsp[-2].minor.yy0));
	drange_node_set_length(yylhsminor.yy29, stnode_value(yymsp[0].minor.yy0));

	stnode_free(yymsp[-2].minor.yy0);
	stnode_free(yymsp[0].minor.yy0);
}
#line 1079 "grammar.c"
  yy_destructor(yypParser,21,&yymsp[-1].minor);
  yymsp[-2].minor.yy29 = yylhsminor.yy29;
        break;
      case 20: /* drnode ::= INTEGER HYPHEN INTEGER */
#line 222 "grammar.lemon"
{
	yylhsminor.yy29 = drange_node_new();
	drange_node_set_start_offset(yylhsminor.yy29, stnode_value(yymsp[-2].minor.yy0));
	drange_node_set_end_offset(yylhsminor.yy29, stnode_value(yymsp[0].minor.yy0));

	stnode_free(yymsp[-2].minor.yy0);
	stnode_free(yymsp[0].minor.yy0);
}
#line 1093 "grammar.c"
  yy_destructor(yypParser,22,&yymsp[-1].minor);
  yymsp[-2].minor.yy29 = yylhsminor.yy29;
        break;
      case 21: /* drnode ::= COLON INTEGER */
{  yy_destructor(yypParser,21,&yymsp[-1].minor);
#line 234 "grammar.lemon"
{
	yymsp[-1].minor.yy29 = drange_node_new();
	drange_node_set_start_offset(yymsp[-1].minor.yy29, 0);
	drange_node_set_length(yymsp[-1].minor.yy29, stnode_value(yymsp[0].minor.yy0));

	stnode_free(yymsp[0].minor.yy0);
}
#line 1107 "grammar.c"
}
        break;
      case 22: /* drnode ::= INTEGER COLON */
#line 244 "grammar.lemon"
{
	yylhsminor.yy29 = drange_node_new();
	drange_node_set_start_offset(yylhsminor.yy29, stnode_value(yymsp[-1].minor.yy0));
	drange_node_set_to_the_end(yylhsminor.yy29);

	stnode_free(yymsp[-1].minor.yy0);
}
#line 1119 "grammar.c"
  yy_destructor(yypParser,21,&yymsp[0].minor);
  yymsp[-1].minor.yy29 = yylhsminor.yy29;
        break;
      case 23: /* drnode ::= INTEGER */
#line 254 "grammar.lemon"
{
	yylhsminor.yy29 = drange_node_new();
	drange_node_set_start_offset(yylhsminor.yy29, stnode_value(yymsp[0].minor.yy0));
	drange_node_set_length(yylhsminor.yy29, 1);

	stnode_free(yymsp[0].minor.yy0);
}
#line 1132 "grammar.c"
  yymsp[0].minor.yy29 = yylhsminor.yy29;
        break;
      case 24: /* relation_test ::= entity rel_op2 entity */
#line 266 "grammar.lemon"
{
	yylhsminor.yy0 = stnode_new(STTYPE_TEST, NULL);
	sttype_test_set2(yylhsminor.yy0, yymsp[-1].minor.yy2, yymsp[-2].minor.yy0, yymsp[0].minor.yy0);
}
#line 1141 "grammar.c"
  yymsp[-2].minor.yy0 = yylhsminor.yy0;
        break;
      case 25: /* relation_test ::= entity rel_op2 relation_test */
#line 273 "grammar.lemon"
{
	stnode_t *L, *F;
	/* for now generate it like yymsp[-2].minor.yy0 yymsp[-1].minor.yy2 F  TEST_OP_AND  F P G, later it could be optimized
	   or semantically checked (to make a <= b >= c or a == b != c invalid)?
	 */

	F = yymsp[0].minor.yy0;
	do {
		g_assert(F != NULL && stnode_type_id(F) == STTYPE_TEST);
		sttype_test_get(F, NULL, &F, NULL);
	} while (stnode_type_id(F) == STTYPE_TEST);

	L = stnode_new(STTYPE_TEST, NULL);
	sttype_test_set2(L, yymsp[-1].minor.yy2, yymsp[-2].minor.yy0, stnode_dup(F));

	yylhsminor.yy0 = stnode_new(STTYPE_TEST, NULL);
	sttype_test_set2(yylhsminor.yy0, TEST_OP_AND, L, yymsp[0].minor.yy0);
}
#line 1164 "grammar.c"
  yymsp[-2].minor.yy0 = yylhsminor.yy0;
        break;
      case 26: /* rel_op2 ::= TEST_EQ */
{  yy_destructor(yypParser,3,&yymsp[0].minor);
#line 292 "grammar.lemon"
{ yymsp[0].minor.yy2 = TEST_OP_EQ; }
#line 1171 "grammar.c"
}
        break;
      case 27: /* rel_op2 ::= TEST_NE */
{  yy_destructor(yypParser,4,&yymsp[0].minor);
#line 293 "grammar.lemon"
{ yymsp[0].minor.yy2 = TEST_OP_NE; }
#line 1178 "grammar.c"
}
        break;
      case 28: /* rel_op2 ::= TEST_GT */
{  yy_destructor(yypParser,7,&yymsp[0].minor);
#line 294 "grammar.lemon"
{ yymsp[0].minor.yy2 = TEST_OP_GT; }
#line 1185 "grammar.c"
}
        break;
      case 29: /* rel_op2 ::= TEST_GE */
{  yy_destructor(yypParser,8,&yymsp[0].minor);
#line 295 "grammar.lemon"
{ yymsp[0].minor.yy2 = TEST_OP_GE; }
#line 1192 "grammar.c"
}
        break;
      case 30: /* rel_op2 ::= TEST_LT */
{  yy_destructor(yypParser,5,&yymsp[0].minor);
#line 296 "grammar.lemon"
{ yymsp[0].minor.yy2 = TEST_OP_LT; }
#line 1199 "grammar.c"
}
        break;
      case 31: /* rel_op2 ::= TEST_LE */
{  yy_destructor(yypParser,6,&yymsp[0].minor);
#line 297 "grammar.lemon"
{ yymsp[0].minor.yy2 = TEST_OP_LE; }
#line 1206 "grammar.c"
}
        break;
      case 32: /* rel_op2 ::= TEST_BITWISE_AND */
{  yy_destructor(yypParser,11,&yymsp[0].minor);
#line 298 "grammar.lemon"
{ yymsp[0].minor.yy2 = TEST_OP_BITWISE_AND; }
#line 1213 "grammar.c"
}
        break;
      case 33: /* rel_op2 ::= TEST_CONTAINS */
{  yy_destructor(yypParser,9,&yymsp[0].minor);
#line 299 "grammar.lemon"
{ yymsp[0].minor.yy2 = TEST_OP_CONTAINS; }
#line 1220 "grammar.c"
}
        break;
      case 34: /* rel_op2 ::= TEST_MATCHES */
{  yy_destructor(yypParser,10,&yymsp[0].minor);
#line 300 "grammar.lemon"
{ yymsp[0].minor.yy2 = TEST_OP_MATCHES; }
#line 1227 "grammar.c"
}
        break;
      case 35: /* relation_test ::= entity TEST_IN LBRACE setnode_list RBRACE */
#line 303 "grammar.lemon"
{
	stnode_t *S;
	yylhsminor.yy0 = stnode_new(STTYPE_TEST, NULL);
	S = stnode_new(STTYPE_SET, yymsp[-1].minor.yy81);
	sttype_test_set2(yylhsminor.yy0, TEST_OP_IN, yymsp[-4].minor.yy0, S);
}
#line 1238 "grammar.c"
  yy_destructor(yypParser,23,&yymsp[-3].minor);
  yy_destructor(yypParser,24,&yymsp[-2].minor);
  yy_destructor(yypParser,25,&yymsp[0].minor);
  yymsp[-4].minor.yy0 = yylhsminor.yy0;
        break;
      case 36: /* setnode_list ::= entity */
      case 40: /* funcparams ::= entity */ yytestcase(yyruleno==40);
#line 311 "grammar.lemon"
{
	yylhsminor.yy81 = g_slist_append(NULL, yymsp[0].minor.yy0);
}
#line 1250 "grammar.c"
  yymsp[0].minor.yy81 = yylhsminor.yy81;
        break;
      case 37: /* setnode_list ::= setnode_list entity */
#line 316 "grammar.lemon"
{
	yylhsminor.yy81 = g_slist_append(yymsp[-1].minor.yy81, yymsp[0].minor.yy0);
}
#line 1258 "grammar.c"
  yymsp[-1].minor.yy81 = yylhsminor.yy81;
        break;
      case 38: /* entity ::= FUNCTION LPAREN funcparams RPAREN */
#line 324 "grammar.lemon"
{
	yylhsminor.yy0 = yymsp[-3].minor.yy0;
	sttype_function_set_params(yylhsminor.yy0, yymsp[-1].minor.yy81);
}
#line 1267 "grammar.c"
  yy_destructor(yypParser,27,&yymsp[-2].minor);
  yy_destructor(yypParser,28,&yymsp[0].minor);
  yymsp[-3].minor.yy0 = yylhsminor.yy0;
        break;
      case 39: /* entity ::= FUNCTION LPAREN RPAREN */
#line 331 "grammar.lemon"
{
	yylhsminor.yy0 = yymsp[-2].minor.yy0;
}
#line 1277 "grammar.c"
  yy_destructor(yypParser,27,&yymsp[-1].minor);
  yy_destructor(yypParser,28,&yymsp[0].minor);
  yymsp[-2].minor.yy0 = yylhsminor.yy0;
        break;
      case 41: /* funcparams ::= funcparams COMMA entity */
#line 341 "grammar.lemon"
{
	yylhsminor.yy81 = g_slist_append(yymsp[-2].minor.yy81, yymsp[0].minor.yy0);
}
#line 1287 "grammar.c"
  yy_destructor(yypParser,19,&yymsp[-1].minor);
  yymsp[-2].minor.yy81 = yylhsminor.yy81;
        break;
      case 42: /* expr ::= LPAREN expr RPAREN */
{  yy_destructor(yypParser,27,&yymsp[-2].minor);
#line 348 "grammar.lemon"
{
	yymsp[-2].minor.yy0 = yymsp[-1].minor.yy0;
	stnode_set_bracket(yymsp[-2].minor.yy0, TRUE);
}
#line 1298 "grammar.c"
  yy_destructor(yypParser,28,&yymsp[0].minor);
}
        break;
      default:
        break;
/********** End reduce actions ************************************************/
  };
  assert( yyruleno<sizeof(yyRuleInfo)/sizeof(yyRuleInfo[0]) );
  yygoto = yyRuleInfo[yyruleno].lhs;
  yysize = yyRuleInfo[yyruleno].nrhs;
  yyact = yy_find_reduce_action(yymsp[-yysize].stateno,(YYCODETYPE)yygoto);
  if( yyact <= YY_MAX_SHIFTREDUCE ){
    if( yyact>YY_MAX_SHIFT ){
      yyact += YY_MIN_REDUCE - YY_MIN_SHIFTREDUCE;
    }
    yymsp -= yysize-1;
    yypParser->yytos = yymsp;
    yymsp->stateno = (YYACTIONTYPE)yyact;
    yymsp->major = (YYCODETYPE)yygoto;
    yyTraceShift(yypParser, yyact);
  }else{
    assert( yyact == YY_ACCEPT_ACTION );
    yypParser->yytos -= yysize;
    yy_accept(yypParser);
  }
}

/*
** The following code executes when the parse fails
*/
#ifndef YYNOERRORRECOVERY
static void yy_parse_failed(
  yyParser *yypParser           /* The parser */
){
  DfilterARG_FETCH;
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sFail!\n",yyTracePrompt);
  }
#endif
  while( yypParser->yytos>yypParser->yystack ) yy_pop_parser_stack(yypParser);
  /* Here code is inserted which will be executed whenever the
  ** parser fails */
/************ Begin %parse_failure code ***************************************/
#line 130 "./grammar.lemon"

	dfw->syntax_error = TRUE;
#line 1346 "grammar.c"
/************ End %parse_failure code *****************************************/
  DfilterARG_STORE; /* Suppress warning about unused %extra_argument variable */
}
#endif /* YYNOERRORRECOVERY */

/*
** The following code executes when a syntax error first occurs.
*/
static void yy_syntax_error(
  yyParser *yypParser,           /* The parser */
  int yymajor _U_,               /* The major type of the error token */
  DfilterTOKENTYPE yyminor         /* The minor type of the error token */
){
  DfilterARG_FETCH;
#define TOKEN yyminor
/************ Begin %syntax_error code ****************************************/
#line 70 "./grammar.lemon"


	header_field_info	*hfinfo;

	if (!TOKEN) {
		dfilter_fail(dfw, "Unexpected end of filter string.");
		dfw->syntax_error = TRUE;
		return;
	}

	switch(stnode_type_id(TOKEN)) {
		case STTYPE_UNINITIALIZED:
			dfilter_fail(dfw, "Syntax error.");
			break;
		case STTYPE_TEST:
			dfilter_fail(dfw, "Syntax error, TEST.");
			break;
		case STTYPE_STRING:
			dfilter_fail(dfw, "The string \"%s\" was unexpected in this context.",
				(char *)stnode_data(TOKEN));
			break;
		case STTYPE_CHARCONST:
			dfilter_fail(dfw, "The character constant \"%s\" was unexpected in this context.",
				(char *)stnode_data(TOKEN));
			break;
		case STTYPE_UNPARSED:
			dfilter_fail(dfw, "\"%s\" was unexpected in this context.",
				(char *)stnode_data(TOKEN));
			break;
		case STTYPE_INTEGER:
			dfilter_fail(dfw, "The integer %d was unexpected in this context.",
				stnode_value(TOKEN));
			break;
		case STTYPE_FIELD:
			hfinfo = (header_field_info *)stnode_data(TOKEN);
			dfilter_fail(dfw, "Syntax error near \"%s\".", hfinfo->abbrev);
			break;
		case STTYPE_FUNCTION:
			dfilter_fail(dfw, "The function s was unexpected in this context.");
			break;
		case STTYPE_SET:
			dfilter_fail(dfw, "Syntax error, SET.");
			break;

		/* These aren't handed to use as terminal tokens from
		   the scanner, so was can assert that we'll never
		   see them here. */
		case STTYPE_NUM_TYPES:
		case STTYPE_RANGE:
		case STTYPE_FVALUE:
			g_assert_not_reached();
			break;
	}
	dfw->syntax_error = TRUE;
#line 1418 "grammar.c"
/************ End %syntax_error code ******************************************/
  DfilterARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/*
** The following is executed when the parser accepts
*/
static void yy_accept(
  yyParser *yypParser           /* The parser */
){
  DfilterARG_FETCH;
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sAccept!\n",yyTracePrompt);
  }
#endif
#ifndef YYNOERRORRECOVERY
  yypParser->yyerrcnt = -1;
#endif
  assert( yypParser->yytos==yypParser->yystack );
  /* Here code is inserted which will be executed whenever the
  ** parser accepts */
/*********** Begin %parse_accept code *****************************************/
/*********** End %parse_accept code *******************************************/
  DfilterARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/* The main parser program.
** The first argument is a pointer to a structure obtained from
** "DfilterAlloc" which describes the current state of the parser.
** The second argument is the major token number.  The third is
** the minor token.  The fourth optional argument is whatever the
** user wants (and specified in the grammar) and is available for
** use by the action routines.
**
** Inputs:
** <ul>
** <li> A pointer to the parser (an opaque structure.)
** <li> The major token number.
** <li> The minor token number.
** <li> An option argument of a grammar-specified type.
** </ul>
**
** Outputs:
** None.
*/
void Dfilter(
  void *yyp,                   /* The parser */
  int yymajor,                 /* The major token code number */
  DfilterTOKENTYPE yyminor       /* The value for the token */
  DfilterARG_PDECL               /* Optional %extra_argument parameter */
){
  YYMINORTYPE yyminorunion;
  unsigned int yyact;   /* The parser action. */
#if !defined(YYERRORSYMBOL) && !defined(YYNOERRORRECOVERY)
  int yyendofinput;     /* True if we are at the end of input */
#endif
#ifdef YYERRORSYMBOL
  int yyerrorhit = 0;   /* True if yymajor has invoked an error */
#endif
  yyParser *yypParser;  /* The parser */

  yypParser = (yyParser*)yyp;
  assert( yypParser->yytos!=0 );
#if !defined(YYERRORSYMBOL) && !defined(YYNOERRORRECOVERY)
  yyendofinput = (yymajor==0);
#endif
  DfilterARG_STORE;

#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sInput '%s'\n",yyTracePrompt,yyTokenName[yymajor]);
  }
#endif

  do{
    yyact = yy_find_shift_action(yypParser,(YYCODETYPE)yymajor);
    if( yyact <= YY_MAX_SHIFTREDUCE ){
      yy_shift(yypParser,yyact,yymajor,yyminor);
#ifndef YYNOERRORRECOVERY
      yypParser->yyerrcnt--;
#endif
      yymajor = YYNOCODE;
    }else if( yyact <= YY_MAX_REDUCE ){
      yy_reduce(yypParser,yyact-YY_MIN_REDUCE);
    }else{
      assert( yyact == YY_ERROR_ACTION );
      yyminorunion.yy0 = yyminor;
#ifdef YYERRORSYMBOL
      int yymx;
#endif
#ifndef NDEBUG
      if( yyTraceFILE ){
        fprintf(yyTraceFILE,"%sSyntax Error!\n",yyTracePrompt);
      }
#endif
#ifdef YYERRORSYMBOL
      /* A syntax error has occurred.
      ** The response to an error depends upon whether or not the
      ** grammar defines an error token "ERROR".
      **
      ** This is what we do if the grammar does define ERROR:
      **
      **  * Call the %syntax_error function.
      **
      **  * Begin popping the stack until we enter a state where
      **    it is legal to shift the error symbol, then shift
      **    the error symbol.
      **
      **  * Set the error count to three.
      **
      **  * Begin accepting and shifting new tokens.  No new error
      **    processing will occur until three tokens have been
      **    shifted successfully.
      **
      */
      if( yypParser->yyerrcnt<0 ){
        yy_syntax_error(yypParser,yymajor,yyminor);
      }
      yymx = yypParser->yytos->major;
      if( yymx==YYERRORSYMBOL || yyerrorhit ){
#ifndef NDEBUG
        if( yyTraceFILE ){
          fprintf(yyTraceFILE,"%sDiscard input token %s\n",
             yyTracePrompt,yyTokenName[yymajor]);
        }
#endif
        yy_destructor(yypParser, (YYCODETYPE)yymajor, &yyminorunion);
        yymajor = YYNOCODE;
      }else{
        while( yypParser->yytos >= yypParser->yystack
            && yymx != YYERRORSYMBOL
            && (yyact = yy_find_reduce_action(
                        yypParser->yytos->stateno,
                        YYERRORSYMBOL)) >= YY_MIN_REDUCE
        ){
          yy_pop_parser_stack(yypParser);
        }
        if( yypParser->yytos < yypParser->yystack || yymajor==0 ){
          yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
          yy_parse_failed(yypParser);
#ifndef YYNOERRORRECOVERY
          yypParser->yyerrcnt = -1;
#endif
          yymajor = YYNOCODE;
        }else if( yymx!=YYERRORSYMBOL ){
          yy_shift(yypParser,yyact,YYERRORSYMBOL,yyminor);
        }
      }
      yypParser->yyerrcnt = 3;
      yyerrorhit = 1;
#elif defined(YYNOERRORRECOVERY)
      /* If the YYNOERRORRECOVERY macro is defined, then do not attempt to
      ** do any kind of error recovery.  Instead, simply invoke the syntax
      ** error routine and continue going as if nothing had happened.
      **
      ** Applications can set this macro (for example inside %include) if
      ** they intend to abandon the parse upon the first syntax error seen.
      */
      yy_syntax_error(yypParser,yymajor, yyminor);
      yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
      yymajor = YYNOCODE;

#else  /* YYERRORSYMBOL is not defined */
      /* This is what we do if the grammar does not define ERROR:
      **
      **  * Report an error message, and throw away the input token.
      **
      **  * If the input token is $, then fail the parse.
      **
      ** As before, subsequent error messages are suppressed until
      ** three input tokens have been successfully shifted.
      */
      if( yypParser->yyerrcnt<=0 ){
        yy_syntax_error(yypParser,yymajor, yyminor);
      }
      yypParser->yyerrcnt = 3;
      yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
      if( yyendofinput ){
        yy_parse_failed(yypParser);
#ifndef YYNOERRORRECOVERY
        yypParser->yyerrcnt = -1;
#endif
      }
      yymajor = YYNOCODE;
#endif
    }
  }while( yymajor!=YYNOCODE && yypParser->yytos>yypParser->yystack );
#ifndef NDEBUG
  if( yyTraceFILE ){
    yyStackEntry *i;
    char cDiv = '[';
    fprintf(yyTraceFILE,"%sReturn. Stack=",yyTracePrompt);
    for(i=&yypParser->yystack[1]; i<=yypParser->yytos; i++){
      fprintf(yyTraceFILE,"%c%s", cDiv, yyTokenName[i->major]);
      cDiv = ' ';
    }
    fprintf(yyTraceFILE,"]\n");
  }
#endif
  return;
}
